c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine reference_discretization(Ns,Nk,spectra,
     &     Nk_ref,nu_ref)
      implicit none
      include 'max.inc'
c
c     Purpose: to parse a number of input spectral discretization arrays 
c     in order to find the reference discretization that will be
c     adequate to each individual spectrum
c
c     Input:
c       + Ns: number of individual spectra
c       + Nk: number of points for each spectrum
c       + spectra: array of spectra
c     
c     Output:
c       + Nk_ref: number of points in the reference nu array
c       + nu_ref: reference nu array
c

c     I/O
      integer Ns
      integer Nk(1:Nfiles_mx)
      double precision spectra(1:Nk_mx,1:2,1:Nfiles_mx)
      integer Nk_ref
      double precision nu_ref(1:Nk_mx)
c     temp
      integer ik,i,j,is,Nks
      double precision nu_inf,nu_sup
      double precision spectrum(1:Nk_mx,1:2)
      integer Nints
      integer int_indexs(1:Nint_mx,1:2)
      double precision int_nus(1:Nint_mx,1:2)
      integer Nint(1:Nfiles_mx)
      integer int_index(1:Nint_mx,1:2,1:Nfiles_mx)
      double precision int_nu(1:Nint_mx,1:2,1:Nfiles_mx)
      double precision int_dnu(1:Nint_mx,1:Nfiles_mx)
      double precision dnu_min
      integer Nint_ref,out
      double precision int_nu_ref(1:Nint_mx,1:2)
      integer Np_ref(1:Nint_mx)
      integer interval(1:Nfiles_mx)
      integer is_ref,interval_ref
      double precision nu_hi(1:Nfiles_mx)
      double precision nu0,nu_hi_min
      double precision delta_nu,av_dnu,dnu
      character*(Nchar_mx) outfile
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine reference_discretization'

      nu_inf=spectra(1,1,1)
      nu_sup=spectra(Nk(1),1,1)
c     Debug
c      write(*,*) 'nu_inf=',nu_inf
c      write(*,*) 'nu_sup=',nu_sup
c     Debug

c     computing intervals
      do is=1,Ns
         Nks=Nk(is)
         do ik=1,Nks
            do j=1,2
               spectrum(ik,j)=spectra(ik,j,is)
            enddo               ! j
         enddo                  !ik
         call intervals(Nks,spectrum,Nints,int_indexs,int_nus)
c     Debug
c         write(*,*) 'Nints(',is,')=',Nints
c         do i=1,Nints
c            write(*,*) int_nus(i,1),int_nus(i,2)
c         enddo
c     Debug
         Nint(is)=Nints
         do i=1,Nints
            do j=1,2
               int_index(i,j,is)=int_indexs(i,j)
               int_nu(i,j,is)=int_nus(i,j)
            enddo ! j
         enddo ! i
      enddo ! is
c     computing discretization steps for each interval
      do is=1,Ns
         do i=1,Nint(is)
            int_dnu(i,is)=(int_nu(i,2,is)-int_nu(i,1,is))
     &           /dble(int_index(i,2,is)-int_index(i,1,is))
         enddo ! i
      enddo ! is

      Nint_ref=0
      nu0=nu_inf
      out=0
      do while (out.eq.0)
c     identify intervals where 'nu0' is located
         do is=1,Ns
            Nints=Nint(is)
            do i=1,Nints
               do j=1,2
                  int_indexs(i,j)=int_index(i,j,is)
                  int_nus(i,j)=int_nu(i,j,is)
               enddo            ! j
            enddo               ! i
            call identify_interval(nu0,Nints,int_nus,
     &           interval(is),nu_hi(is))
         enddo                  ! is
c     identify the interval whose end is the closest from 'nu0'
         is_ref=1
         nu_hi_min=nu_hi(is_ref)
         interval_ref=interval(is_ref)
         do is=1,Ns
            if (nu_hi(is).lt.nu_hi_min) then
               nu_hi_min=nu_hi(is)
               is_ref=is
               interval_ref=interval(is)
            endif
         enddo                  ! is
c     Debug
c         write(*,*) 'Nint_ref=',Nint_ref,' nu0=',nu0
c     Debug
c     identify the smaller discretization step around "nu0"
         dnu_min=int_dnu(interval(1),1)
         do is=1,Ns
            if (int_dnu(interval(is),is).lt.dnu_min) then
               dnu_min=int_dnu(interval(is),is)
            endif
         enddo ! is
         
c     add current interval to the list of intervals in the reference discretization
         Nint_ref=Nint_ref+1
         if (Nint_ref.gt.Nint_mx) then
            call error(label)
            write(*,*) 'Nint_ref=',Nint_ref
            write(*,*) 'while Nint_mx=',Nint_mx
            stop
         endif
         int_nu_ref(Nint_ref,1)=nu0
         int_nu_ref(Nint_ref,2)=nu_hi_min
c         av_dnu=(int_nu(interval_ref,2,is_ref)
c     &        -int_nu(interval_ref,1,is_ref))/
c             dble(int_index(interval_ref,2,is_ref)
c     &        -int_index(interval_ref,1,is_ref))
         delta_nu=(int_nu_ref(Nint_ref,2)-int_nu_ref(Nint_ref,1))
         if (delta_nu.lt.dnu_min) then
            Np_ref(Nint_ref)=1
         else
            Np_ref(Nint_ref)=int(delta_nu/dnu_min)
         endif
c     Debug
         if (Np_ref(Nint_ref).le.0) then
            call error(label)
            write(*,*) 'Np_ref(',Nint_ref,')=',Np_ref(Nint_ref)
            write(*,*) 'int_nu_ref(',Nint_ref,',1)=',
     &           int_nu_ref(Nint_ref,1)
            write(*,*) 'int_nu_ref(',Nint_ref,',2)=',
     &           int_nu_ref(Nint_ref,2)
            write(*,*) 'av_dnu=',av_dnu
            stop
         endif
c     Debug
         nu0=nu_hi_min
         if (nu_hi_min.eq.nu_sup) then
            out=1
         endif
         if (Nint_ref.eq.Nint_mx) then
            out=1
         endif
      enddo ! while out=0
c     Debug
c      write(*,*) 'Nint_ref=',Nint_ref
c      do i=1,1
c         write(*,*) 'i=',i
c         write(*,*) int_nu_ref(i,1),int_nu_ref(i,2),Np_ref(i)
c      enddo
c     Debug

c     Production of the output reference discretization
      Nk_ref=1
      nu_ref(Nk_ref)=nu_inf
      do i=1,Nint_ref
         dnu=(int_nu_ref(i,2)-int_nu_ref(i,1))/dble(Np_ref(i))
         do j=1,Np_ref(i)
            Nk_ref=Nk_ref+1
            nu_ref(Nk_ref)=int_nu_ref(i,1)+dnu*j
c     stop if Nk_mx points are already in the reference discretization array
            if (Nk_ref.eq.Nk_mx) then
               goto 666
            endif
         enddo ! j
c     security for (infinitely small) numerical errors
         if (nu_ref(Nk_ref).ne.int_nu_ref(i,2)) then
            nu_ref(Nk_ref)=int_nu_ref(i,2)
         endif
      enddo ! i

 666  continue

c     Debug
c      do is=1,Ns
c         write(*,*) 'Nk(',is,')=',Nk(is)
c      enddo
c      write(*,*) 'Nk_ref=',Nk_ref
c      write(*,*) 'nu_ref(1)=',nu_ref(1)
c     write(*,*) 'nu_ref(',Nk_ref,')=',nu_ref(Nk_ref)
      outfile='./results/reference_discretization.txt'
      open(10,file=trim(outfile))
      do i=1,Nk_ref
         write(10,*) nu_ref(i)
      enddo ! i
      close(10)
      write(*,*) 'File was generated: ',trim(outfile)
c      stop
c     Debug

      return
      end



      subroutine identify_interval(nu,Nint,int_nu,
     &     interval,nu_hi)
      implicit none
      include 'max.inc'
c
c     Purpose: to identify the interval in which "nu" is located
c
c     Input:
c       + nu: wavenumber that has to be located
c       + Nint: number of intervals
c       + int_nu: limits of each interval in terms of wavenumber
c     
c     Output:
c       + interval: index of the interval in which "nu" is located
c       + nu_hi: upper wavenumber of the interval
c

c     I/O
      double precision nu
      integer Nint
      double precision int_nu(1:Nint_mx,1:2)
      integer interval
      double precision nu_hi
c     temp
      integer i,intf
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine identify_interval'

      intf=0
      do i=1,Nint
         if ((nu.ge.int_nu(i,1)).and.(nu.le.int_nu(i,2))) then
            intf=1
            interval=i
            goto 111
         endif
      enddo ! i
 111  continue
      if (intf.eq.0) then
         call error(label)
         write(*,*) 'nu=',nu
         write(*,*) 'could not be located in spectrum:'
         write(*,*) 'int_nu(1,1)=',int_nu(1,1)
         write(*,*) 'int_nu(',Nint,',2)=',int_nu(Nint,2)
         stop
      endif
      if ((nu.eq.int_nu(interval,2)).and.(interval.lt.Nint)) then
         interval=interval+1
      endif
      
c     Debug
      if (interval.lt.1) then
         call error(label)
         write(*,*) 'interval=',interval
         write(*,*) 'is lower than 1'
         stop
      endif
      if (interval.gt.Nint) then
         call error(label)
         write(*,*) 'interval=',interval
         write(*,*) 'is greater than Nint=',Nint
         stop
      endif
c     Debug
      nu_hi=int_nu(interval,2)

      return
      end



      subroutine intervals(Nk,spectrum,Nint,int_index,int_nu)
      implicit none
      include 'max.inc'
c
c     Purpose: to identify witin a given spectrum the nu-intervals such that the
c     average spectral discretization changes by an order of magnitude from
c     an interval to the next one.
c
c     Input:
c       + Nk: number of values of nu/k(nu) for the provided spectrum
c       + spectrum: the Nk values of nu/k(nu)
c
c     Output:
c       + Nint: number of intervals
c       + int_index: indexes of begin and end of each interval
c       + int_nu: nu at the beginning and at the end of each interval
c

c     I/O
      integer Nk
      double precision spectrum(1:Nk_mx,1:2)
      integer Nint
      integer int_index(1:Nint_mx,1:2)
      double precision int_nu(1:Nint_mx,1:2)
c     temp
      integer ik,jk,out,jk_end,jk_endf
      double precision dnu0,dnu
      double precision sf,endofnu
      parameter(sf=10.0D+0)
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine intervals'
      
      Nint=0
      ik=1
      out=0
c     Debug
c      write(*,*) 'Nk=',Nk
c     Debug
      do while (out.eq.0)
         Nint=Nint+1
         if (Nint.gt.Nint_mx) then
            call error(label)
            write(*,*) 'Nint=',Nint
            write(*,*) 'while Nint_mx=',Nint_mx
            stop
         endif
         int_index(Nint,1)=ik
         int_nu(Nint,1)=spectrum(ik,1)
c     Debug
c         write(*,*) 'Nint=',Nint
c         write(*,*) 'nu(',int_index(Nint,1),')=',int_nu(Nint,1)
c     Debug
         dnu0=spectrum(ik+1,1)-spectrum(ik,1)
c     Debug
c         write(*,*) 'dnu0=',dnu0
c     Debug
         if (ik.eq.Nk-1) then
            int_index(Nint,2)=Nk
            int_nu(Nint,2)=spectrum(Nk,1)
            out=1
            goto 111
         endif
         jk_endf=0
         endofnu=0
         do jk=ik+1,Nk-1
            dnu=spectrum(jk+1,1)-spectrum(jk,1)
c     Debug
c            write(*,*) 'jk=',jk,' dnu=',dnu
c     Debug
            if ((dnu.le.dnu0/sf).or.(dnu.ge.dnu0*sf)) then
               jk_end=jk
               jk_endf=1
               goto 222
            endif
c     Debug
c            write(*,*) 'jk=',jk,jk_endf
c     Debug
            if (jk.eq.Nk-1) then
               endofnu=1
            endif
         enddo ! jk
 222     continue
c     Debug
c            write(*,*) 'output, jk=',jk,endofnu
c     Debug
         if ((jk_endf.eq.0).and.(endofnu.eq.1)) then
            jk_endf=1
            jk_end=Nk
         endif
         if (jk_endf.eq.0) then
            call error(label)
            write(*,*) 'jk_endf=',jk_endf
            write(*,*) 'jk=',jk
            stop
         endif
         ik=jk_end
         int_index(Nint,2)=ik
         int_nu(Nint,2)=spectrum(ik,1)
c     Debug
c         write(*,*) 'nu(',int_index(Nint,2),')=',int_nu(Nint,2)
c     Debug
         if (ik.eq.Nk) then
            out=1
         endif
 111     continue
      enddo ! while out=0

      return
      end

      
