c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine spectrum_files(hs_base,Nf,files,nu_start,nu_end)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to identify the number of files that contain
c     high-resolution spectrum for the atmosphere
c
c     Input:
c       + hs_base: folder that contains high-resolution spectrum
c         files for the atmosphere.
c
c     Output:
c       + Nf: number of files (is also the index of the last file)
c         that contain high-resolution spectrum.
c       + files: names of these Nf files
c       + nu_start: first wavenumber defined in these Nf files
c       + nu_end: last wavenumber defined in these files
c
c     I/O
      character*(Nchar_mx) hs_base
      integer Nf
      character*(Nchar_mx) files(1:Nfiles_mx)
      double precision nu_start,nu_end
c     temp
      character*(Nchar_mx) kfile,nlines_file,nu_file
      integer ios,i,nl,mol,n,Nk,Nb,band
      double precision nu_low(1:Nb_mx)
      double precision nu_hi(1:Nb_mx)
      double precision P,T
      integer Nmol
      character*(Nchar_mx) mol_name(1:Nmol_mx)
      double precision x(1:Nmol_mx)
      double precision P_ref,T_ref
      integer Nmol_ref
      character*(Nchar_mx) mol_name_ref(1:Nmol_mx)
      double precision x_ref(1:Nmol_mx)
      logical keep_going
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine spectrum_files'

      call add_slash_if_absent(hs_base)
      i=0
      Nf=0
      keep_going=.true.
      nlines_file='./nlines'
      write(*,*) 'Locating files within:'
      write(*,*) hs_base(1:strlen(hs_base))
      do while (keep_going)
         i=i+1
         call generate_kfile(hs_base,i,kfile)
         call get_nlines(kfile,nlines_file,nl)
c     Debug
c         write(*,*) kfile(1:strlen(kfile))
c         write(*,*) 'nl=',nl
c     Debug
         if (nl.ne.0) then ! file exists
            Nf=Nf+1
            if (Nf.ge.Nfiles_mx) then
               call error(label)
               write(*,*) 'Nf=',Nf
               write(*,*) 'while Nfiles_mx=',Nfiles_mx
               stop
            endif
            files(Nf)=kfile(1:strlen(kfile))
         else ! file not found
            keep_going=.false.
         endif
      enddo
c     "Nf" is the number of files that have been identified
c     "files" is the array that contains the names of these "Nf" files

c     Debug
c      write(*,*) 'Nf=',Nf
c      do i=1,Nf
c         write(*,*) files(i)(1:strlen(files(i)))
c      enddo ! i
c     Debug

c     Analysing high-resolution spectrum files
      write(*,*) 'Analysing',Nf,' files...'
      do i=1,Nf
         call read_header(files(i),P,T,Nmol,mol_name,x)
         if (i.eq.1) then
            P_ref=P
            T_ref=T
            Nmol_ref=Nmol
            do mol=1,Nmol_ref
               mol_name_ref(mol)=mol_name(mol)
               x_ref(mol)=x(mol)
            enddo ! mol
         else
c            if (P.ne.P_ref) then
c               call error(label)
c               write(*,*) 'while analysing file:'
c               write(*,*) files(i)(1:strlen(files(i)))
c               write(*,*) 'P=',P
c               write(*,*) 'differs from previous conditions'
c               stop
c            endif
c            if (T.ne.T_ref) then
c               call error(label)
c               write(*,*) 'while analysing file:'
c               write(*,*) files(i)(1:strlen(files(i)))
c               write(*,*) 'T=',T
c               write(*,*) 'differs from previous conditions'
c               stop
c            endif
            if (Nmol.ne.Nmol_ref) then
               call error(label)
               write(*,*) 'while analysing file:'
               write(*,*) files(i)(1:strlen(files(i)))
               write(*,*) 'Nmol=',Nmol
               write(*,*) 'differs from previous conditions'
               stop
            endif
            do mol=1,Nmol
               if (mol_name(mol)(1:strlen(mol_name(mol))).ne.
     &              mol_name_ref(mol)(1:strlen(mol_name_ref(mol))))
     &              then
                  call error(label)
                  write(*,*) 'while analysing file:'
                  write(*,*) files(i)(1:strlen(files(i)))
                  write(*,*) 'mol_name(',mol,')=',
     &                 mol_name(mol)(1:strlen(mol_name(mol)))
                  write(*,*) 'differs from previous conditions'
                  stop
               endif
c               if (x(mol).ne.x_ref(mol)) then
c                  call error(label)
c                  write(*,*) 'while analysing file:'
c                  write(*,*) files(i)(1:strlen(files(i)))
c                  write(*,*) 'x(',mol,')=',x(mol)
c                  write(*,*) 'differs from previous conditions'
c                  stop
c               endif
            enddo ! mol
         endif ! i=1
      enddo ! i
      write(*,*) 'Number of high-resolution spectrum files:',Nf

c     wavenumber limits
      nu_file=hs_base(1:strlen(hs_base))
     &     //'narowbands.in'
      open(11,file=nu_file(1:strlen(nu_file)),status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         call get_nlines(files(1),nlines_file,nl)
         open(12,file=files(1)(1:strlen(files(1))))
         do i=1,5
            read(12,*)
         enddo                  ! i
         read(12,*) Nmol
         n=5*Nmol+3
         Nk=nl-(n+6)
         do i=1,n
            read(12,*)
         enddo                  ! i
         read(12,52) nu_start
         do i=1,Nk-2
            read(12,*)
         enddo                  ! i
         read(12,52) nu_end
         close(12)
      else
         read(11,*) Nb
         do band=1,Nb
            read(12,*) nu_low(band),nu_hi(band)
         enddo ! band
         nu_start=nu_low(1)
         nu_end=nu_hi(Nb)
      endif
      close(11)

      return
      end



      subroutine read_header(kfile,P,T,Nmol,mol_name,x)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to read a kspectrum output file's header
c
c     Input:
c       + kfile: path to the file whose header has to be read
c     
c     Output:
c       + P: pressure (atm)
c       + T: temperature (K)
c       + Nmol: number of molecular species
c       + mol_name: array of names of the Nmol molecular species
c       + x: molar fractions of the Nmol molecular species
c

c     I/O
      character*(Nchar_mx) kfile
      double precision P,T
      integer Nmol
      character*(Nchar_mx) mol_name(1:Nmol_mx)
      double precision x(1:Nmol_mx)
c     temp
      integer ios,mol
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine read_header'

      open(11,file=kfile(1:strlen(kfile)),status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         call error(label)
         write(*,*) 'File could not be opened:'
         write(*,*) kfile(1:strlen(kfile))
         stop
      else
         read(11,10) !'Pressure (atm):'
         read(11,50) P
         read(11,10) !'Temperature (K):'
         read(11,50) T
         read(11,10) !'Number of molecules:'
         read(11,*) Nmol
         read(11,*)
         do mol=1,Nmol
            read(11,23) !'Molecule index:',mol
            read(11,10) !'Name:'
            read(11,10) mol_name(mol)
            read(11,10) !'Concentration:'
            read(11,50) x(mol)
         enddo ! mol
      endif
      close(11)

      return
      end


      subroutine nullify_tau_array(Nk,Nlay,tau)
      implicit none
      include 'max.inc'
c     
c     Purpose: to initialize a given optical thickness array
c     
c     Input:
c       + Nk: number of points for each spectrum
c       + Nlay: number of layers
c       + tau: array to initialize
c     
c     Output:
c       + tau: updated
c     
c     I/O
      integer Nk,Nlay
      double precision tau(1:Nk_mx,1:Nlay_mx)
c     temp
      integer ik,ilay
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine nullify_tau_arrau'

      do ik=1,Nk
         do ilay=1,Nlay
            tau(ik,ilay)=0.0D+0
         enddo                  ! ilay
      enddo                     ! ik

      return
      end
      
