c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine find_free_process(free,nproc,fpf,proc)
      implicit none
      include 'max.inc'
c
c     Purpose: to find 'proc', the index of a free child process
c
c     Input:
c       + free: 0/1 value for each child process (0: busy process, 1: free process)
c       + nproc: number of child processes
c
c     Output:
c       + fpf: 0 if no free child process found; 1 otherwise
c       + proc: index of the free child process found, if fpf=1
c
c     I/O
      integer free(1:Nproc_mx)
      integer proc,nproc,fpf
c     temp
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine find_free_process'

      fpf=0
      do proc=1,nproc
         if (free(proc).eq.1) then
            fpf=1
            goto 123
         endif
      enddo
 123  continue

      return
      end



      subroutine stopped_processes(free,nproc,nsp)
      implicit none
      include 'max.inc'
c
c     Purpose: to identify the number of child processes that have been stopped
c
c     Input:
c       + free: array of free processes (a value of 2 means that the
c         corresponding child process has been stopped)
c       + nproc: number of child processes
c     
c     Output:
c       + nsp: number of child processes that have been stopped
c     
c     I/O
      integer free(1:Nproc_mx)
      integer proc,nproc,nsp
c     temp
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine stopped_processes'

      nsp=0
      do proc=1,nproc
         if (free(proc).eq.2) then
            nsp=nsp+1
         endif
      enddo

      return
      end


      subroutine set_Nchunks(Nk,Nchunks)
      implicit none
      include 'max.inc'
c
c     Purpose: to set the number of computation chunks
c
c     Input:
c       + Nk: number of nk(k) values RT has to be held for
c
c     Output:
c       + Nchunks: number of computation chunks
c
c     I/O
      integer Nk,Nchunks
c     temp
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine set_Nchunks'

      Nchunks=Nk

      return
      end


      
      subroutine send_global_data(ai,mu,use_cloud,
     &     add_sca2abs,cner,cflux,
     &     ref,Nlev,Nlay,Nk,
     &     alt,pres,temp,Tg,Ts,epsilon_ground,abs_cont,ks_coeff,
     &     probe_pressure,probe_layer,probe_altitude,
     &     nu_ref,spectra_rt,
     &     pure_co2,x_co2,Nmol,mol_name,x,
     &     N2N2_ndt,N2N2_np,N2N2_nu_min,N2N2_nu_max,
     &     N2N2_T,N2N2_cia,
     &     O2O2_ndt,O2O2_np,O2O2_nu_min,O2O2_nu_max,
     &     O2O2_T,O2O2_cia)
      implicit none
      include 'mpif.h'
      include 'max.inc'

c     I/O
      logical ai
      double precision mu
      logical use_cloud,add_sca2abs,cner,cflux
      integer ref,Nlev,Nlay,Nk
      integer probe_layer
      double precision alt(1:Nlev_mx)
      double precision pres(1:Nlev_mx)
      double precision temp(1:Nlev_mx)
      double precision Tg,Ts
      double precision epsilon_ground
      double precision abs_cont
      double precision ks_coeff
      double precision probe_pressure
      double precision probe_altitude
      double precision nu_ref(1:Nk_mx)
      double precision spectra_rt(1:Nk_mx,1:2,1:Nfiles_mx)
      logical pure_co2
      double precision x_co2(1:Nlev_mx)
      integer Nmol
      character*10 mol_name(1:Nmol_mx)
      double precision x(1:Nlev_mx,1:Nmol_mx)
      integer N2N2_ndt,N2N2_np(1:Ndt_int_mx)
      double precision N2N2_nu_min(1:Ndt_int_mx)
      double precision N2N2_nu_max(1:Ndt_int_mx)
      double precision N2N2_T(1:Ndt_int_mx)
      double precision N2N2_cia(1:Ndt_int_mx,1:Np_mx,1:2)
      integer O2O2_ndt,O2O2_np(1:Ndt_int_mx)
      double precision O2O2_nu_min(1:Ndt_int_mx)
      double precision O2O2_nu_max(1:Ndt_int_mx)
      double precision O2O2_T(1:Ndt_int_mx)
      double precision O2O2_cia(1:Ndt_int_mx,1:Np_mx,1:2)
c     temp
      integer i,j,k
      double precision spectra_tmp(1:Nk_mx)
c     MPICH
      integer ierr
      integer stat(MPI_STATUS_SIZE)
c     MPICH
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine send_global_data'

c     LOGICAL data
      call MPI_BCAST(ai,1,MPI_LOGICAL
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(use_cloud,1,MPI_LOGICAL
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(add_sca2abs,1,MPI_LOGICAL
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(cner,1,MPI_LOGICAL
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(cflux,1,MPI_LOGICAL
     &     ,0,MPI_COMM_WORLD,ierr)
c     INTEGER data
      call MPI_BCAST(ref,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nlev,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nlay,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nk,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(probe_layer,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
c     DOUBLE PRECISION data
      call MPI_BCAST(mu,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(alt,Nlev_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(pres,Nlev_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(temp,Nlev_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Tg,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Ts,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(epsilon_ground,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(abs_cont,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(ks_coeff,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(probe_pressure,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(probe_altitude,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(nu_ref,Nk_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      do i=1,2
         do j=1,Nfiles_mx
            do k=1,Nk_mx
               spectra_tmp(k)=spectra_rt(k,i,j)
            enddo ! k
            call MPI_BCAST(spectra_tmp,Nk_mx,MPI_DOUBLE_PRECISION
     &           ,0,MPI_COMM_WORLD,ierr)
         enddo                  ! j
      enddo                     ! i
      call MPI_BCAST(pure_co2,1,MPI_LOGICAL
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(x_co2,Nlev_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nmol,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(mol_name,10*Nmol_mx,MPI_BYTE
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(x,Nlev_mx*Nmol_mx,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(N2N2_ndt,1,
     &     MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(N2N2_np,Ndt_int_mx,
     &     MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(N2N2_nu_min,Ndt_int_mx,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(N2N2_nu_max,Ndt_int_mx,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(N2N2_T,Ndt_int_mx,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(N2N2_cia,Ndt_int_mx*Np_mx*2,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(O2O2_ndt,1,
     &     MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(O2O2_np,Ndt_int_mx,
     &     MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(O2O2_nu_min,Ndt_int_mx,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(O2O2_nu_max,Ndt_int_mx,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(O2O2_T,Ndt_int_mx,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(O2O2_cia,Ndt_int_mx*Np_mx*2,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)

      return
      end


      
      subroutine send_opacities(tau_rayleigh,tau_co2_cia,tau_h2o_cia,
     &     tau_N2N2_cia,tau_O2O2_cia,tau_h2o_cont,
     &     Nb_cloud,nu_lo_cloud,nu_hi_cloud,ka_cloud,ks_cloud,g_cloud)
      implicit none
      include 'mpif.h'
      include 'max.inc'
c     I/O
      double precision tau_rayleigh(1:Nk_mx,1:Nlay_mx)
      double precision tau_co2_cia(1:Nk_mx,1:Nlay_mx)
      double precision tau_h2o_cia(1:Nk_mx,1:Nlay_mx)
      double precision tau_N2N2_cia(1:Nk_mx,1:Nlay_mx)
      double precision tau_O2O2_cia(1:Nk_mx,1:Nlay_mx)
      double precision tau_h2o_cont(1:Nk_mx,1:Nlay_mx)
      integer Nb_cloud
      double precision nu_lo_cloud(1:Nb_mx)
      double precision nu_hi_cloud(1:Nb_mx)
      double precision ka_cloud(1:Nlev_mx,1:Nb_mx)
      double precision ks_cloud(1:Nlev_mx,1:Nb_mx)
      double precision g_cloud(1:Nlev_mx,1:Nb_mx)
c     MPICH
      integer ierr
      integer stat(MPI_STATUS_SIZE)
c     MPICH
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine send_opacities'

      call MPI_BCAST(tau_rayleigh,Nk_mx*Nlay_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(tau_co2_cia,Nk_mx*Nlay_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(tau_h2o_cia,Nk_mx*Nlay_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(tau_N2N2_cia,Nk_mx*Nlay_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(tau_O2O2_cia,Nk_mx*Nlay_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(tau_h2o_cont,Nk_mx*Nlay_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nb_cloud,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(nu_lo_cloud,Nb_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(nu_hi_cloud,Nb_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(ka_cloud,Nlev_mx*Nb_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(ks_cloud,Nlev_mx*Nb_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(g_cloud,Nlev_mx*Nb_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)

      return
      end
      

      
      subroutine receive_global_data(ai,mu,use_cloud,
     &     add_sca2abs,cner,cflux,
     &     ref,Nlev,Nlay,Nk,
     &     alt,pres,temp,Tg,Ts,epsilon_ground,abs_cont,ks_coeff,
     &     probe_pressure,probe_layer,probe_altitude,
     &     nu_ref,spectra_rt,
     &     pure_co2,x_co2,Nmol,mol_name,x,
     &     N2N2_ndt,N2N2_np,N2N2_nu_min,N2N2_nu_max,
     &     N2N2_T,N2N2_cia,
     &     O2O2_ndt,O2O2_np,O2O2_nu_min,O2O2_nu_max,
     &     O2O2_T,O2O2_cia)
      implicit none
      include 'mpif.h'
      include 'max.inc'

c     I/O
      logical ai
      double precision mu
      logical use_cloud,add_sca2abs,cner,cflux
      integer ref,Nlev,Nlay,Nk
      double precision alt(1:Nlev_mx)
      double precision pres(1:Nlev_mx)
      double precision temp(1:Nlev_mx)
      double precision Tg,Ts
      double precision epsilon_ground
      double precision abs_cont
      double precision ks_coeff
      double precision probe_pressure
      integer probe_layer
      double precision probe_altitude
      double precision nu_ref(1:Nk_mx)
      double precision spectra_rt(1:Nk_mx,1:2,1:Nfiles_mx)
      logical pure_co2
      double precision x_co2(1:Nlev_mx)
      integer Nmol
      character*10 mol_name(1:Nmol_mx)
      double precision x(1:Nlev_mx,1:Nmol_mx)
      integer N2N2_ndt,N2N2_np(1:Ndt_int_mx)
      double precision N2N2_nu_min(1:Ndt_int_mx)
      double precision N2N2_nu_max(1:Ndt_int_mx)
      double precision N2N2_T(1:Ndt_int_mx)
      double precision N2N2_cia(1:Ndt_int_mx,1:Np_mx,1:2)
      integer O2O2_ndt,O2O2_np(1:Ndt_int_mx)
      double precision O2O2_nu_min(1:Ndt_int_mx)
      double precision O2O2_nu_max(1:Ndt_int_mx)
      double precision O2O2_T(1:Ndt_int_mx)
      double precision O2O2_cia(1:Ndt_int_mx,1:Np_mx,1:2)
c     temp
      integer i,j,k
      double precision spectra_tmp(1:Nk_mx)
c     MPICH
      integer ierr
      integer stat(MPI_STATUS_SIZE)
c     MPICH
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine receive_global_data'

c     LOGICAL data
      call MPI_BCAST(ai,1,MPI_LOGICAL
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(use_cloud,1,MPI_LOGICAL
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(add_sca2abs,1,MPI_LOGICAL
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(cner,1,MPI_LOGICAL
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(cflux,1,MPI_LOGICAL
     &     ,0,MPI_COMM_WORLD,ierr)
c     INTEGER data
      call MPI_BCAST(ref,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nlev,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nlay,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nk,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(probe_layer,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
c     DOUBLE PRECISION data
      call MPI_BCAST(mu,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(alt,Nlev_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(pres,Nlev_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(temp,Nlev_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Tg,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Ts,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(epsilon_ground,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(abs_cont,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(ks_coeff,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(probe_pressure,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(probe_altitude,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(nu_ref,Nk_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      do i=1,2
         do j=1,Nfiles_mx
            call MPI_BCAST(spectra_tmp,Nk_mx,MPI_DOUBLE_PRECISION
     &           ,0,MPI_COMM_WORLD,ierr)
            do k=1,Nk_mx
               spectra_rt(k,i,j)=spectra_tmp(k)
            enddo ! k
         enddo ! j
      enddo ! i
      call MPI_BCAST(pure_co2,1,MPI_LOGICAL
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(x_co2,Nlev_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nmol,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(mol_name,10*Nmol_mx,MPI_BYTE
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(x,Nlev_mx*Nmol_mx,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(N2N2_ndt,1,
     &     MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(N2N2_np,Ndt_int_mx,
     &     MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(N2N2_nu_min,Ndt_int_mx,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(N2N2_nu_max,Ndt_int_mx,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(N2N2_T,Ndt_int_mx,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(N2N2_cia,Ndt_int_mx*Np_mx*2,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(O2O2_ndt,1,
     &     MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(O2O2_np,Ndt_int_mx,
     &     MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(O2O2_nu_min,Ndt_int_mx,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(O2O2_nu_max,Ndt_int_mx,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(O2O2_T,Ndt_int_mx,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(O2O2_cia,Ndt_int_mx*Np_mx*2,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)

      return
      end
      

      
      subroutine receive_opacities(tau_rayleigh,tau_co2_cia,tau_h2o_cia,
     &     tau_N2N2_cia,tau_O2O2_cia,tau_h2o_cont,
     &     Nb_cloud,nu_lo_cloud,nu_hi_cloud,ka_cloud,ks_cloud,g_cloud)
      implicit none
      include 'mpif.h'
      include 'max.inc'

c     I/O
      double precision tau_rayleigh(1:Nk_mx,1:Nlay_mx)
      double precision tau_co2_cia(1:Nk_mx,1:Nlay_mx)
      double precision tau_h2o_cia(1:Nk_mx,1:Nlay_mx)
      double precision tau_N2N2_cia(1:Nk_mx,1:Nlay_mx)
      double precision tau_O2O2_cia(1:Nk_mx,1:Nlay_mx)
      double precision tau_h2o_cont(1:Nk_mx,1:Nlay_mx)
      integer Nb_cloud
      double precision nu_lo_cloud(1:Nb_mx)
      double precision nu_hi_cloud(1:Nb_mx)
      double precision ka_cloud(1:Nlev_mx,1:Nb_mx)
      double precision ks_cloud(1:Nlev_mx,1:Nb_mx)
      double precision g_cloud(1:Nlev_mx,1:Nb_mx)
c     MPICH
      integer ierr
      integer stat(MPI_STATUS_SIZE)
c     MPICH
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine receive_opacities'

      call MPI_BCAST(tau_rayleigh,Nk_mx*Nlay_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(tau_co2_cia,Nk_mx*Nlay_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(tau_h2o_cia,Nk_mx*Nlay_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(tau_N2N2_cia,Nk_mx*Nlay_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(tau_O2O2_cia,Nk_mx*Nlay_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(tau_h2o_cont,Nk_mx*Nlay_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nb_cloud,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(nu_lo_cloud,Nb_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(nu_hi_cloud,Nb_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(ka_cloud,Nlev_mx*Nb_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(ks_cloud,Nlev_mx*Nb_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(g_cloud,Nlev_mx*Nb_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      
      return
      end
      


      subroutine send_partial_results(ner,flux_up,flux_dn,flux_net,
     &     pflux_up,pflux_dn,pflux_net)
      implicit none
      include 'mpif.h'
      include 'max.inc'
c     I/O
      double precision ner(0:Nlay_mx+1,0:Nlay_mx+1)
      double precision flux_up(0:Nlay_mx)
      double precision flux_dn(0:Nlay_mx)
      double precision flux_net(0:Nlay_mx)
      double precision pflux_up
      double precision pflux_dn
      double precision pflux_net
c     MPICH
      integer ierr
      integer stat(MPI_STATUS_SIZE)
c     MPICH
c     temp
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine send_partial_results'

      call MPI_SEND(ner,(Nlay_mx+2)*(Nlay_mx+2),
     &     MPI_DOUBLE_PRECISION,0,1,
     &     MPI_COMM_WORLD,ierr)
      call MPI_SEND(flux_up,Nlay_mx+1,
     &     MPI_DOUBLE_PRECISION,0,1,
     &     MPI_COMM_WORLD,ierr)
      call MPI_SEND(flux_dn,Nlay_mx+1,
     &     MPI_DOUBLE_PRECISION,0,1,
     &     MPI_COMM_WORLD,ierr)
      call MPI_SEND(flux_net,Nlay_mx+1,
     &     MPI_DOUBLE_PRECISION,0,1,
     &     MPI_COMM_WORLD,ierr)
      call MPI_SEND(pflux_up,1,
     &     MPI_DOUBLE_PRECISION,0,1,
     &     MPI_COMM_WORLD,ierr)
      call MPI_SEND(pflux_dn,1,
     &     MPI_DOUBLE_PRECISION,0,1,
     &     MPI_COMM_WORLD,ierr)
      call MPI_SEND(pflux_net,1,
     &     MPI_DOUBLE_PRECISION,0,1,
     &     MPI_COMM_WORLD,ierr)
      
      return
      end



      subroutine receive_partial_results(proc,
     &     ner,flux_up,flux_dn,flux_net,
     &     pflux_up,pflux_dn,pflux_net)
      implicit none
      include 'mpif.h'
      include 'max.inc'
c     I/O
      double precision ner(0:Nlay_mx+1,0:Nlay_mx+1)
      double precision flux_up(0:Nlay_mx)
      double precision flux_dn(0:Nlay_mx)
      double precision flux_net(0:Nlay_mx)
      double precision pflux_up
      double precision pflux_dn
      double precision pflux_net
c     MPICH
      integer proc,ierr
      integer stat(MPI_STATUS_SIZE)
c     MPICH
c     temp
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine receive_partial_results'

      call MPI_RECV(ner,(Nlay_mx+2)*(Nlay_mx+2),
     &     MPI_DOUBLE_PRECISION,proc,
     &     MPI_ANY_TAG,MPI_COMM_WORLD,
     &     stat,ierr)
      call MPI_RECV(flux_up,Nlay_mx+1,
     &     MPI_DOUBLE_PRECISION,proc,
     &     MPI_ANY_TAG,MPI_COMM_WORLD,
     &     stat,ierr)
      call MPI_RECV(flux_dn,Nlay_mx+1,
     &     MPI_DOUBLE_PRECISION,proc,
     &     MPI_ANY_TAG,MPI_COMM_WORLD,
     &     stat,ierr)
      call MPI_RECV(flux_net,Nlay_mx+1,
     &     MPI_DOUBLE_PRECISION,proc,
     &     MPI_ANY_TAG,MPI_COMM_WORLD,
     &     stat,ierr)
      call MPI_RECV(pflux_up,1,
     &     MPI_DOUBLE_PRECISION,proc,
     &     MPI_ANY_TAG,MPI_COMM_WORLD,
     &     stat,ierr)
      call MPI_RECV(pflux_dn,1,
     &     MPI_DOUBLE_PRECISION,proc,
     &     MPI_ANY_TAG,MPI_COMM_WORLD,
     &     stat,ierr)
      call MPI_RECV(pflux_net,1,
     &     MPI_DOUBLE_PRECISION,proc,
     &     MPI_ANY_TAG,MPI_COMM_WORLD,
     &     stat,ierr)
      
      return
      end
