c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine read_cloud_data(cloud_base,Nlev,Nb_cloud,
     &     nu_lo_cloud,nu_hi_cloud,ka_cloud,ks_cloud,g_cloud)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: read all relevant cloud data
c
c     Input:
c       + cloud_base: base directory for cloud data
c       + Nlev: number of atmospheric levels
c
c     Output:
c       + Nb_cloud: number of cloud spectral intervals
c       + nu_lo_cloud: lower wavenumber of each cloud interval (inv. cm)
c       + nu_hi_cloud: lower wavenumber of each cloud interval (inv. cm)
c       + ka_cloud: cloud absorbtion coefficient (inv. m) for each interval, for each level
c       + ks_cloud: cloud scattering coefficient (inv. m) for each interval, for each level
c       + g_cloud: cloud asymetry parameter for each interval, for each level
c
c     I/O
      character*(Nchar_mx) cloud_base
      integer Nlev
      integer Nb_cloud
      double precision nu_lo_cloud(1:Nb_mx)
      double precision nu_hi_cloud(1:Nb_mx)
      double precision ka_cloud(1:Nlev_mx,1:Nb_mx)
      double precision ks_cloud(1:Nlev_mx,1:Nb_mx)
      double precision g_cloud(1:Nlev_mx,1:Nb_mx)
c     temp
      integer ios
      integer ilev,band
      character*(Nchar_mx) infile
      character*3 ilev_str3
      double precision raf1,raf2
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine read_cloud_data'

c     Reading spectral interval limits
      call add_slash_if_absent(cloud_base)
      infile=cloud_base(1:strlen(cloud_base))
     &     //'narrowbands.in'
      open(14,file=infile(1:strlen(infile)),
     &        status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         call error(label)
         write(*,*) 'Data file could not be found:'
         write(*,*) infile(1:strlen(infile))
         stop
      else
         read(14,*) Nb_cloud
         if (Nb_cloud.gt.Nb_mx) then
            call error(label)
            write(*,*) 'Nb_cloud=',Nb_cloud
            write(*,*) 'while Nb_mx=',Nb_mx
            stop
         endif
         do band=1,Nb_cloud
            read(14,*) nu_lo_cloud(band),nu_hi_cloud(band)
         enddo ! band
      endif
      close(14)

c     Reading absorption coefficient
      do ilev=1,Nlev
         call num2str3(ilev,ilev_str3)
         infile=cloud_base(1:strlen(cloud_base))
     &        //'ka_cloud_'
     &        //ilev_str3(1:strlen(ilev_str3))
         open(15,file=infile(1:strlen(infile)),
     &        status='old',iostat=ios)
         if (ios.ne.0) then     ! file not found
            call error(label)
            write(*,*) 'Data file could not be found:'
            write(*,*) infile(1:strlen(infile))
            stop
         else
            do band=1,Nb_cloud
               read(15,52) raf1,raf2,ka_cloud(ilev,band)
            enddo ! band
         endif
         close(15)
c     Reading scattering coefficient
         infile=cloud_base(1:strlen(cloud_base))
     &        //'ks_cloud_'
     &        //ilev_str3(1:strlen(ilev_str3))
         open(16,file=infile(1:strlen(infile)),
     &        status='old',iostat=ios)
         if (ios.ne.0) then     ! file not found
            call error(label)
            write(*,*) 'Data file could not be found:'
            write(*,*) infile(1:strlen(infile))
            stop
         else
            do band=1,Nb_cloud
               read(16,52) raf1,raf2,ks_cloud(ilev,band)
            enddo ! band
         endif
         close(16)
c     Reading asymetry parameter
         infile=cloud_base(1:strlen(cloud_base))
     &        //'g_cloud_'
     &        //ilev_str3(1:strlen(ilev_str3))
         open(17,file=infile(1:strlen(infile)),
     &        status='old',iostat=ios)
         if (ios.ne.0) then     ! file not found
            call error(label)
            write(*,*) 'Data file could not be found:'
            write(*,*) infile(1:strlen(infile))
            stop
         else
            do band=1,Nb_cloud
               read(17,52) raf1,raf2,g_cloud(ilev,band)
            enddo ! band
         endif
         close(17)
      enddo ! ilev

      return
      end

