c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine read_options(ai,use_cloud,
     &     compute_rayleigh,compute_co2_cia,compute_h2o_cia,
     &     compute_N2N2_cia,compute_O2O2_cia,
     &     compute_h2o_aer_cont,add_sca2abs,
     &     cner,cflux,ref,
     &     record_rayleigh,record_co2_cia,record_h2o_cia,
     &     record_N2N2_cia,record_O2O2_cia,record_h2o_aer_cont,
     &     record_total_ot,record_asr,test_data)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: read the "options.in" file
c
c     Input:
c
c     Output:
c       + ai: true if angular integration is enabled
c       + use_cloud: if true, cloud data will be used
c       + compute_rayleigh: if true, Rayleigh optical thickness is computed
c       + compute_co2_cia: if true, CO2 CIA optical thickness is computed
c       + compute_h2o_cia: if true, H2O CIA optical thickness is computed
c       + compute_N2N2_cia: if true, N2-N2 CIA optical thickness is computed
c       + compute_O2O2_cia: if true, N2-N2 CIA optical thickness is computed
c       + compute_h2o_aer_cont: if true, H2O AER continuum optical thickness (due to line wings truncation) is computed
c       + add_sca2abs: if true, scattering is added to the absorption optical thicnkess
c       + cner: T if computation of NER matrix is required; F otherwise
c       + cflux: T if computation of flux profiles is required; F otherwise
c       + ref: option for reflection
c         - ref=1: specular reflection
c       - ref=2: diffuse reflection
c       + record_rayleigh: if true, Rayleigh optical thickness is recorded
c       + record_co2_cia: if true, CO2 CIA optical thickness is recorded
c       + record_h2o_cia: if true, H2O CIA optical thickness is recorded
c       + record_N2N2_cia: if true, N2-N2 CIA optical thickness is recorded
c       + record_O2O2_cia: if true, N2-N2 CIA optical thickness is recorded
c       + record_h2o_aer_cont: if true, H2O AER continuum optical thickness (due to line wings truncation) is recorded
c       + record_total_ot: if true, total optical thicknesses are recorded
c       + record_asr: if true, all spectral results are recorded; otherwise, only selected files are recorded
c       + test_data: if true, every recorded variable will be tested against NaN and INF
c
c     I/O
      logical ai
      logical use_cloud
      logical compute_rayleigh
      logical compute_co2_cia
      logical compute_h2o_cia
      logical compute_N2N2_cia
      logical compute_O2O2_cia
      logical compute_h2o_aer_cont
      logical add_sca2abs
      logical cner,cflux
      logical record_rayleigh
      logical record_co2_cia
      logical record_h2o_cia
      logical record_N2N2_cia
      logical record_O2O2_cia
      logical record_h2o_aer_cont
      logical record_total_ot
      logical record_asr
      logical test_data
      integer ref
c     temp
      character*(Nchar_mx) op_file
      integer ios,i
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine read_options'

      op_file='./options.in'
      open(11,file=op_file(1:strlen(op_file))
     &     ,status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         call error(label)
         write(*,*) 'File not found:'
         write(*,*) op_file(1:strlen(op_file))
         stop
      else
         do i=1,6
            read(11,*)
         enddo
         read(11,*) ai
         read(11,*) 
         read(11,*) ref
         do i=1,5
            read(11,*)
         enddo
         read(11,*) use_cloud
         read(11,*)
         read(11,*) compute_rayleigh
         read(11,*)
         read(11,*) compute_co2_cia
         read(11,*)
         read(11,*) compute_h2o_cia
         read(11,*)
         read(11,*) compute_N2N2_cia
         read(11,*)
         read(11,*) compute_O2O2_cia
         read(11,*)
         read(11,*) compute_h2o_aer_cont
         read(11,*)
         read(11,*) add_sca2abs
         do i=1,5
            read(11,*)
         enddo
         read(11,*) cner
         read(11,*)
         read(11,*) cflux
         do i=1,5
            read(11,*)
         enddo
         read(11,*) record_rayleigh
         read(11,*)
         read(11,*) record_co2_cia
         read(11,*)
         read(11,*) record_h2o_cia
         read(11,*)
         read(11,*) record_N2N2_cia
         read(11,*)
         read(11,*) record_O2O2_cia
         read(11,*)
         read(11,*) record_h2o_aer_cont
         read(11,*)
         read(11,*) record_total_ot
         read(11,*)
         read(11,*) record_asr
         read(11,*)
         read(11,*) test_data
      endif
      close(11)

      if ((ref.ne.1).and.(ref.ne.2)) then
         call error(label)
         write(*,*) 'Bad input parameter:'
         write(*,*) 'ref=',ref
         write(*,*) 'allowed values: 1, 2'
         stop
      endif

      return
      end

