c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine read_kspectra(Nf,files,nu_start,nu_end,
     &     last_nu,Nk,spectra)
      implicit none
      include 'max.inc'
c
c     Purpose: to read the "Nf" files and return the corresponding high-resolution
c     spectra starting from a known wavenumber
c
c     Input:
c       + Nf: number of files to read
c       + files: array of file names to read
c       + nu_start: wavenumber the files have to be read from
c       + nu_end: desired ending wavenumber
c     
c     Output:
c       + last_nu: last wavenumber actually read from files
c       + Nk: number of nu/k(nu) values in each spectrum read
c       + spectra: the "Nf" spectra read from files
c
c     I/O
      integer Nf
      character*(Nchar_mx) files(1:Nfiles_mx)
      double precision nu_start,nu_end
      double precision last_nu
      integer Nk(1:Nfiles_mx)
      double precision spectra(1:Nk_mx,1:2,1:Nfiles_mx)
c     temp
      integer i,j,jj,ik
      double precision nu_hi(1:Nfiles_mx)
      double precision spectrum(1:Nk_mx,1:2)
      double precision nu_hi_tmp
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine read_spectra'
      
c     read spectrum for file 'i'
      last_nu=nu_end
      do i=1,Nf
c     Debug
c         write(*,*) 'files(',i,')=',files(i)
c     Debug
         call read_single_kspectrum(files(i),nu_start,last_nu,
     &        Nk(i),nu_hi(i),spectrum)
c     Debug
c         write(*,*) 'nu_start=',nu_start
c         write(*,*) 'Nk(',i,')=',Nk(i)
c         write(*,*) 'nu_hi(',i,')=',nu_hi(i)
c         write(*,*) spectrum(1,1),spectrum(1,2)
c     Debug
c     store "spectrum" information into the "spectra" array
         do ik=1,Nk(i)
            do jj=1,2
               spectra(ik,jj,i)=spectrum(ik,jj)
            enddo               ! j
         enddo                  ! ik
c     check wavenumber limits
         if (nu_hi(i).lt.last_nu) then
            do j=1,i-1
               call read_single_kspectrum(files(j),nu_start,nu_hi(i),
     &              Nk(j),nu_hi_tmp,spectrum)
               if (nu_hi_tmp.ne.nu_hi(i)) then
                  call error(label)
                  write(*,*) 'nu_hi(',i,')=',nu_hi(i)
                  write(*,*) 'nu_hi_tmp(',j,')=',nu_hi_tmp
                  stop
               endif
               do ik=1,Nk(j)
                  do jj=1,2
                     spectra(ik,jj,j)=spectrum(ik,jj)
                  enddo         ! j
               enddo            ! ik
            enddo               ! j
            last_nu=nu_hi(i)
c         else
c            last_nu=nu_end
         endif                  ! nu_hi(i)<nu_end
      enddo                     ! i
c     Debug
c      write(*,*) 'last_nu=',last_nu
c      do i=1,Nf
c         write(*,*) i,Nk(i),spectra(1,1,i),spectra(Nk(i),1,i)
c      enddo ! i
c     Debug

      return
      end



      subroutine read_single_kspectrum(file_name,nu_start,nu_end,
     &     Nk,nu_hi,spectra)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c     
c     Purpose: to read the spectrum for a single file, starting from a
c     given wavenumber, until:
c     - the limit wavenumber (specified) has been reached or
c     - the file ends or
c     - the maximal number of nu/k has been reached.
c
c     Input:
c       + file_name: name of the file that has to be read
c       + nu_start: starting wavenumber
c       + nu_end: ending wavenumber
c     
c     Output:
c       + Nk: number of nu/k(nu) values that have been read
c       + nu_hi: ending wavenumber for that file
c       + spectra: values of nu/k(nu
c         - spectra(ik,1): wavenumber index ik
c         - spectra(ik,2): k(nu) index ik
c

c     I/O
      character*(Nchar_mx) file_name
      double precision nu_start,nu_end
      integer Nk
      double precision nu_hi
      double precision spectra(1:Nk_mx,1:2)
c     temp
      double precision spectra_tmp(1:Nk_mx,1:2)
      double precision nu_tmp1,nu_tmp2
      double precision nu1,k1,raf
      double precision nua,ka
      double precision nub,kb
      character*(Nchar_mx) nlines_file
      integer n,n0,nl,nkt,nk_left
      integer ios,i,j,ik,ik0,ik0f
      integer Nmol
      double precision k_end
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine read_single_kspectrum'
      
      nlines_file='./nlines'
      call get_nlines(file_name,nlines_file,nl)

c     Debug
c      write(*,*) 'nu_start=',nu_start
c     Debug

      open(13,file=file_name(1:strlen(file_name))
     &     ,status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         call error(label)
         write(*,*) 'File could not be opened:'
         write(*,*) file_name(1:strlen(file_name))
c         stop
      else
         read(13,10) !'Pressure (atm):'
         read(13,50) !P
         read(13,10) !'Temperature (K):'
         read(13,50) !T
         read(13,10) !'Number of molecules:'
         read(13,*) Nmol
c     Debug
c         write(*,*) 'Nmol=',Nmol
c     Debug
c     n is the number of lines to read in order to read the first nu/k(nu) value
         n=5*Nmol+3
c     n0 is the total number of lines at the beginning to the file
c     befor the first nu/k(nu) value
         n0=n+6
c     nkt is the number of nu/k(nu) values within the file
         nkt=nl-n0
         do i=1,n
            read(13,*)
         enddo
         Nk=0
c     reading first couple nu/k(nu)
         read(13,52) nu1,raf,k1
c     Debug
c         write(*,*) nu1,k1
c         stop
c     Debug
         if (nu1.ge.nu_start) then
            Nk=Nk+1
            spectra(Nk,1)=nu1
            spectra(Nk,2)=k1
            nk_left=nkt-1
         else
c     look for "nu_start"
            nu_tmp1=nu1
            ik0f=0
c     Debug
c            write(*,*) 'nkt=',nkt
c     Debug
            do ik=2,nkt
               read(13,52) nu_tmp2
c     Debug
c               write(*,*) 'nu_tmp(',ik,')=',nu_tmp(ik)
c     Debug
               if ((nu_tmp1.lt.nu_start).and.
     &              (nu_tmp2.ge.nu_start)) then
                  ik0f=1
                  ik0=ik
                  goto 111
               endif
               nu_tmp1=nu_tmp2
            enddo ! ik
 111        continue
            if (ik0f.eq.0) then
c     Debug
c               write(*,*) 'ik0 not found'
c     Debug
               goto 666
            else
               rewind(13)
               do i=1,n0+ik0-2
                  read(13,*)
               enddo
               read(13,52) nua,raf,ka ! index ik0-1
               read(13,52) nub,raf,kb ! index ik0
c     Test
               if ((nu_start.lt.nua).or.(nu_start.gt.nub)) then
                  call error(label)
                  write(*,*) 'nua=',nua
                  write(*,*) 'nub=',nub
                  write(*,*) 'nu_start=',nu_start
                  stop
               endif
c     Test
               Nk=Nk+1
               spectra(Nk,1)=nu_start
c     Debug
c               write(*,*) 'nu(',Nk,')=',spectra(Nk,1)
c     Debug
               call linear_interpolation(nua,nub,ka,kb,
     &              spectra(Nk,1),spectra(Nk,2))
               if (nub.gt.nu_start) then
                  Nk=Nk+1
                  spectra(Nk,1)=nub
                  spectra(Nk,2)=kb
c     Debug
c               write(*,*) 'nu(',Nk,')=',spectra(Nk,1)
c               write(*,*) nu_start,nub,(nu_start.gt.nua)
c     Debug
               endif
               nk_left=nkt-ik0
            endif               ! ik0f=0
         endif                  ! nu1 > nu_start
         
c     reading the rest of the file
         do ik=1,nk_left        ! nkt-2: number of nu/k(nu) values left to read
            Nk=Nk+1
            if (Nk.gt.Nk_mx) then
               Nk=Nk_mx
               goto 666
            else
               read(13,52) spectra(Nk,1),raf,spectra(Nk,2)
               if (spectra(Nk,1).gt.nu_end) then
                  call linear_interpolation(spectra(Nk-1,1),
     &                 spectra(Nk,1),spectra(Nk-1,2),spectra(Nk,2),
     &                 nu_end,k_end)
                  spectra(Nk,1)=nu_end
                  spectra(Nk,2)=k_end
                  goto 666
               endif
c     Debug
c               if (Nk.eq.2) then
c                  write(*,*) 'nu(2)=',spectra(2,1)
c               endif
c     Debug
            endif
         enddo

      endif                     ! ios=0

 666  continue
      close(13)

c     remove duplicates (if any) in output spectra
      do ik=1,Nk
         do j=1,2
            spectra_tmp(ik,j)=spectra(ik,j)
         enddo ! j 
      enddo ! ik
      call remove_duplicates(Nk,spectra_tmp)
      do ik=1,Nk
         do j=1,2
            spectra(ik,j)=spectra_tmp(ik,j)
         enddo ! j 
      enddo ! ik

      if (Nk.ne.0) then
         nu_hi=spectra(Nk,1)
      endif

      return
      end


      
      subroutine set_nu_limits(nu_min,nu_max,nu_start,nu_end)
      implicit none
      include 'max.inc'
c
c     Purpose: to analyse required wavenumber limits and set
c     them accordingly when possible
c
c     Input:
c       + nu_min: required lower wavenumber (inverse centimeters)
c       + nu_max: required higher wavenumber (inverse centimeters)
c       + nu_start: available lower wavenumber (inverse centimeters)
c       + nu_end: available higher wavenumber (inverse centimeters)
c
c     Output:
c       + nu_min: lower wavenumber (inverse centimeters) that will actually be used
c       + nu_max: higher wavenumber (inverse centimeters) that will actually be used
c
c     I/O
      double precision nu_min,nu_max
      double precision nu_start,nu_end
c     temp
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine set_nu_limits'

      if (nu_min.gt.nu_end) then
         call error(label)
         write(*,*) 'required lower limit is:',nu_min
         write(*,*) 'spectral data is available up to:',nu_end
         stop
      endif
      if (nu_max.lt.nu_start) then
         call error(label)
         write(*,*) 'required higher limit is:',nu_max
         write(*,*) 'spectral data is available from:',nu_start
         stop
      endif

      if (nu_min.lt.nu_start) then
         nu_min=nu_start
         call warning(label)
         write(*,*) 'minimum wavelength was set to:',nu_min
      endif
      if (nu_max.gt.nu_end) then
         nu_max=nu_end
         call warning(label)
         write(*,*) 'maximum wavelength was set to:',nu_max
      endif

      return
      end
