c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine update_ner(Nlev,ner,dnu,total_ner)
      implicit none
      include 'max.inc'
c
c     Purpose: to update the total NER matrix
c
c     Input:
c       + Nlev: number of levels (interfaces)
c       + ner: current NER matrix (for a given wavenumber)
c       + dnu: wavenumber (inverse centimeters) interval "ner" has been computed for
c       + total_ner: total NER matrix
c
c     Output:
c       + total_ner: (updated) total NER matrix
c
c     I/O
      integer Nlev
      double precision ner(0:Nlay_mx+1,0:Nlay_mx+1)
      double precision dnu
      double precision total_ner(0:Nlay_mx+1,0:Nlay_mx+1)
c     temp
      integer i,j,Nlay
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine update_ner'

      Nlay=Nlev-1
      do i=0,Nlay+1
         do j=0,Nlay+1
            total_ner(i,j)=total_ner(i,j)
     &           +ner(i,j)*dnu
         enddo ! j
      enddo ! i
      
      return
      end



      subroutine update_flux(Nlev,flux_up,flux_dn,flux_net,
     &     pflux_up,pflux_dn,pflux_net,
     &     dnu,
     &     total_flux_up,total_flux_dn,total_flux_net,
     &     total_pflux_up,total_pflux_dn,total_pflux_net)
      implicit none
      include 'max.inc'
c
c     Purpose: to update the total NER matrix
c
c     Input:
c       + Nlev: number of levels (interfaces)
c       + flux_up: current upward flux array (for a given wavenumber)
c       + flux_dn: current downward flux array (for a given wavenumber)
c       + flux_net: current net flux array (for a given wavenumber)
c       + pflux_up: upward flux @ probe pressure (for a given wavenumber)
c       + pflux_dn: downward flux @ probe pressure (for a given wavenumber)
c       + pflux_net: net flux @ probe pressure (for a given wavenumber)
c       + dnu: wavenumber (inverse centimeters) interval flux arrays have been computed for
c       + total_flux_up: total upward fluxes
c       + total_flux_dn: total downward fluxes
c       + total_flux_net: total net fluxes
c
c     Output:
c       + total_flux_up: (updated) total upward fluxes
c       + total_flux_dn: (updated) total downward fluxes
c       + total_flux_net: (updated) total net fluxes
c       + total_pflux_up: (updated) total upward flux @ probe pressure
c       + total_pflux_dn: (updated) total downward flux @ probe pressure
c       + total_pflux_net: (updated) total net flux @ probe pressure
c
c     I/O
      integer Nlev
      double precision flux_up(0:Nlay_mx)
      double precision flux_dn(0:Nlay_mx)
      double precision flux_net(0:Nlay_mx)
      double precision pflux_up,pflux_dn,pflux_net
      double precision dnu
      double precision total_flux_up(0:Nlay_mx)
      double precision total_flux_dn(0:Nlay_mx)
      double precision total_flux_net(0:Nlay_mx)
      double precision total_pflux_up
      double precision total_pflux_dn
      double precision total_pflux_net
c     temp
      integer i,j,Nlay
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine update_ner'

      Nlay=Nlev-1
      do i=0,Nlay
         total_flux_up(i)=total_flux_up(i)
     &        +flux_up(i)*dnu
         total_flux_dn(i)=total_flux_dn(i)
     &        +flux_dn(i)*dnu
         total_flux_net(i)=total_flux_net(i)
     &        +flux_net(i)*dnu
      enddo                     ! i
      total_pflux_up=total_pflux_up+pflux_up*dnu
      total_pflux_dn=total_pflux_dn+pflux_dn*dnu
      total_pflux_net=total_pflux_net+pflux_net*dnu
      
      return
      end


      
      subroutine finalize_results(Nlev,nu_min,nu_max,
     &     total_ner,total_flux_up,total_flux_dn,total_flux_net)
      implicit none
      include 'max.inc'
c
c     Purpose: to divide total results by total wavenumber range
c
c     Input:
c       + Nlev: number of levels (interfaces)
c       + nu_min: first wavenumber (inverse centimeters) RT has been held for
c       + nu_max: last wavenumber (inverse centimeters) RT has been held for
c       + total_ner: total NER matrix
c       + total_flux_up: total upward fluxes
c       + total_flux_dn: total downward fluxes
c       + total_flux_net: total net fluxes
c
c     Output:
c       + total_ner: (updated) total NER matrix
c       + total_flux_up: (updated) total upward fluxes
c       + total_flux_dn: (updated) total downward fluxes
c       + total_flux_net: (updated) total net fluxes
c
c     I/O
      integer Nlev
      double precision nu_min,nu_max
      double precision total_ner(0:Nlay_mx+1,0:Nlay_mx+1)
      double precision total_flux_up(0:Nlay_mx)
      double precision total_flux_dn(0:Nlay_mx)
      double precision total_flux_net(0:Nlay_mx)
c     temp
      integer i,j,Nlay
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine finalize_results'

      Nlay=Nlev-1
      if (nu_min.ge.nu_max) then
         call error(label)
         write(*,*) 'Incorrect input data:'
         write(*,*) 'nu_min=',nu_min
         write(*,*) 'nu_max=',nu_max
         stop
      endif
      do i=0,Nlay+1
         do j=0,Nlay+1
            total_ner(i,j)=total_ner(i,j)/(nu_max-nu_min)
         enddo ! j
      enddo ! i
      do i=0,Nlay
         total_flux_up(i)=total_flux_up(i)/(nu_max-nu_min)
         total_flux_dn(i)=total_flux_dn(i)/(nu_max-nu_min)
         total_flux_net(i)=total_flux_net(i)/(nu_max-nu_min)
      enddo ! i

      return
      end
